@@ -29,15 +29,15 @@ pyodide_config.set_flags(
 
 
 def test_index(
-    selenium_coverage: typing.Any, testserver_http: PyodideServerInfo, has_jspi: bool
+    selenium_coverage: typing.Any, testserver_http: PyodideServerInfo, prefer_jspi: bool
 ) -> None:
     @run_in_pyodide  # type: ignore[misc]
-    def pyodide_test(selenium_coverage, host: str, port: int, has_jspi: bool) -> None:  # type: ignore[no-untyped-def]
+    def pyodide_test(selenium_coverage, host: str, port: int, prefer_jspi: bool) -> None:  # type: ignore[no-untyped-def]
         import urllib3.contrib.emscripten.fetch
         from urllib3.connection import HTTPConnection
         from urllib3.response import BaseHTTPResponse
 
-        assert urllib3.contrib.emscripten.fetch.has_jspi() == has_jspi
+        assert urllib3.contrib.emscripten.fetch.has_jspi() == prefer_jspi
         conn = HTTPConnection(host, port)
         url = f"http://{host}:{port}/"
         conn.request("GET", url)
@@ -59,20 +59,20 @@ def test_index(
         selenium_coverage,
         testserver_http.http_host,
         testserver_http.http_port,
-        has_jspi,
+        prefer_jspi,
     )
 
 
 def test_pool_requests(
-    selenium_coverage: typing.Any, testserver_http: PyodideServerInfo, has_jspi: bool
+    selenium_coverage: typing.Any, testserver_http: PyodideServerInfo, prefer_jspi: bool
 ) -> None:
     @run_in_pyodide  # type: ignore[misc]
-    def pyodide_test(selenium_coverage, host: str, port: int, https_port: int, has_jspi: bool) -> None:  # type: ignore[no-untyped-def]
+    def pyodide_test(selenium_coverage, host: str, port: int, https_port: int, prefer_jspi: bool) -> None:  # type: ignore[no-untyped-def]
         # first with PoolManager
         import urllib3
         import urllib3.contrib.emscripten.fetch
 
-        assert urllib3.contrib.emscripten.fetch.has_jspi() == has_jspi
+        assert urllib3.contrib.emscripten.fetch.has_jspi() == prefer_jspi
 
         http = urllib3.PoolManager()
         resp = http.request("GET", f"http://{host}:{port}/")
@@ -127,13 +127,13 @@ def test_pool_requests(
         testserver_http.http_host,
         testserver_http.http_port,
         testserver_http.https_port,
-        has_jspi,
+        prefer_jspi,
     )
 
 
 # wrong protocol / protocol error etc. should raise an exception of http.client.HTTPException
 def test_wrong_protocol(
-    selenium_coverage: typing.Any, testserver_http: PyodideServerInfo, has_jspi: bool
+    selenium_coverage: typing.Any, testserver_http: PyodideServerInfo
 ) -> None:
     @run_in_pyodide  # type: ignore[misc]
     def pyodide_test(selenium_coverage, host: str, port: int) -> None:  # type: ignore[no-untyped-def]
@@ -154,7 +154,7 @@ def test_wrong_protocol(
 
 # wrong protocol / protocol error etc. should raise an exception of http.client.HTTPException
 def test_bad_method(
-    selenium_coverage: typing.Any, testserver_http: PyodideServerInfo, has_jspi: bool
+    selenium_coverage: typing.Any, testserver_http: PyodideServerInfo
 ) -> None:
     @run_in_pyodide  # type: ignore[misc]
     def pyodide_test(selenium_coverage, host: str, port: int) -> None:  # type: ignore[no-untyped-def]
@@ -175,7 +175,7 @@ def test_bad_method(
 
 # no connection - should raise
 def test_no_response(
-    selenium_coverage: typing.Any, testserver_http: PyodideServerInfo, has_jspi: bool
+    selenium_coverage: typing.Any, testserver_http: PyodideServerInfo
 ) -> None:
     @run_in_pyodide  # type: ignore[misc]
     def pyodide_test(selenium_coverage, host: str, port: int) -> None:  # type: ignore[no-untyped-def]
@@ -193,9 +193,7 @@ def test_no_response(
     pyodide_test(selenium_coverage, testserver_http.http_host, find_unused_port())
 
 
-def test_404(
-    selenium_coverage: typing.Any, testserver_http: PyodideServerInfo, has_jspi: bool
-) -> None:
+def test_404(selenium_coverage: typing.Any, testserver_http: PyodideServerInfo) -> None:
     @run_in_pyodide  # type: ignore[misc]
     def pyodide_test(selenium_coverage, host: str, port: int) -> None:  # type: ignore[no-untyped-def]
         from urllib3.connection import HTTPConnection
@@ -251,10 +249,8 @@ def test_timeout_warning(
 
 @pytest.mark.webworkers
 def test_timeout_in_worker_non_streaming(
-    selenium_coverage: typing.Any,
     testserver_http: PyodideServerInfo,
     run_from_server: ServerRunnerInfo,
-    has_jspi: bool,
 ) -> None:
     worker_code = f"""
         from urllib3.exceptions import TimeoutError
@@ -283,10 +279,8 @@ def test_timeout_in_worker_non_streaming(
 
 @pytest.mark.webworkers
 def test_timeout_in_worker_streaming(
-    selenium_coverage: typing.Any,
     testserver_http: PyodideServerInfo,
     run_from_server: ServerRunnerInfo,
-    has_jspi: bool,
 ) -> None:
     worker_code = f"""
         import urllib3.contrib.emscripten.fetch
@@ -308,15 +302,20 @@ def test_timeout_in_worker_streaming(
     run_from_server.run_webworker(worker_code)
 
 
+@pytest.mark.parametrize(
+    "connection_cls", ["HTTPSConnection", "VerifiedHTTPSConnection"]
+)
 def test_index_https(
-    selenium_coverage: typing.Any, testserver_http: PyodideServerInfo, has_jspi: bool
+    selenium_coverage: typing.Any,
+    testserver_http: PyodideServerInfo,
+    connection_cls: str,
 ) -> None:
     @run_in_pyodide  # type: ignore[misc]
-    def pyodide_test(selenium_coverage, host: str, port: int) -> None:  # type: ignore[no-untyped-def]
-        from urllib3.connection import HTTPSConnection
+    def pyodide_test(selenium_coverage, host: str, port: int, connection_cls: str) -> None:  # type: ignore[no-untyped-def]
+        from urllib3 import connection
         from urllib3.response import BaseHTTPResponse
 
-        conn = HTTPSConnection(host, port)
+        conn = getattr(connection, connection_cls)(host, port)
         conn.request("GET", f"https://{host}:{port}/")
         response = conn.getresponse()
         assert isinstance(response, BaseHTTPResponse)
@@ -324,7 +323,10 @@ def test_index_https(
         assert data.decode("utf-8") == "Dummy server!"
 
     pyodide_test(
-        selenium_coverage, testserver_http.http_host, testserver_http.https_port
+        selenium_coverage,
+        testserver_http.http_host,
+        testserver_http.https_port,
+        connection_cls,
     )
 
 
@@ -415,8 +417,6 @@ def test_streaming_fallback_warning(
 def test_specific_method(
     selenium_coverage: typing.Any,
     testserver_http: PyodideServerInfo,
-    run_from_server: ServerRunnerInfo,
-    has_jspi: bool,
 ) -> None:
     @run_in_pyodide  # type: ignore[misc]
     def pyodide_test(selenium_coverage, host: str, port: int) -> None:  # type: ignore[no-untyped-def]
@@ -437,10 +437,9 @@ def test_specific_method(
 
 @pytest.mark.webworkers
 def test_streaming_download(
-    selenium_coverage: typing.Any,
     testserver_http: PyodideServerInfo,
     run_from_server: ServerRunnerInfo,
-    has_jspi: bool,
+    prefer_jspi: bool,
 ) -> None:
     # test streaming download, which must be in a webworker
     # as you can't do it on main thread
@@ -461,7 +460,7 @@ def test_streaming_download(
             response = conn.getresponse()
             assert isinstance(response, BaseHTTPResponse)
             assert urllib3.contrib.emscripten.fetch._SHOWN_STREAMING_WARNING==False
-            assert(urllib3.contrib.emscripten.fetch.has_jspi() == {has_jspi})
+            assert(urllib3.contrib.emscripten.fetch.has_jspi() == {prefer_jspi})
             data=response.data.decode('utf-8')
             assert len(data) == 17825792
 """
@@ -470,10 +469,9 @@ def test_streaming_download(
 
 @pytest.mark.webworkers
 def test_streaming_close(
-    selenium_coverage: typing.Any,
     testserver_http: PyodideServerInfo,
     run_from_server: ServerRunnerInfo,
-    has_jspi: bool,
+    prefer_jspi: bool,
 ) -> None:
     # test streaming download, which must be in a webworker
     # as you can't do it on main thread
@@ -497,7 +495,7 @@ def test_streaming_close(
             assert(body_internal.writable() is False)
             assert(body_internal.seekable() is False)
             assert(body_internal.readable() is True)
-            assert(urllib3.contrib.emscripten.fetch.has_jspi() == {has_jspi})
+            assert(urllib3.contrib.emscripten.fetch.has_jspi() == {prefer_jspi})
 
             response.drain_conn()
             x=response.read()
@@ -515,10 +513,8 @@ def test_streaming_close(
 
 @pytest.mark.webworkers
 def test_streaming_bad_url(
-    selenium_coverage: typing.Any,
     testserver_http: PyodideServerInfo,
     run_from_server: ServerRunnerInfo,
-    has_jspi: bool,
 ) -> None:
     # this should cause an error
     # because the protocol is bad
@@ -542,10 +538,8 @@ def test_streaming_bad_url(
 
 @pytest.mark.webworkers
 def test_streaming_bad_method(
-    selenium_coverage: typing.Any,
     testserver_http: PyodideServerInfo,
     run_from_server: ServerRunnerInfo,
-    has_jspi: bool,
 ) -> None:
     # this should cause an error
     # because the protocol is bad
@@ -572,7 +566,6 @@ def test_streaming_bad_method(
 @pytest.mark.webworkers
 @pytest.mark.without_jspi
 def test_streaming_notready_warning(
-    selenium_coverage: typing.Any,
     testserver_http: PyodideServerInfo,
     run_from_server: ServerRunnerInfo,
 ) -> None:
@@ -607,7 +600,7 @@ def test_streaming_notready_warning(
 
 
 def test_post_receive_json(
-    selenium_coverage: typing.Any, testserver_http: PyodideServerInfo, has_jspi: bool
+    selenium_coverage: typing.Any, testserver_http: PyodideServerInfo
 ) -> None:
     @run_in_pyodide  # type: ignore[misc]
     def pyodide_test(selenium_coverage, host: str, port: int) -> None:  # type: ignore[no-untyped-def]
@@ -680,7 +673,6 @@ def test_streaming_not_ready_in_browser(
 def test_requests_with_micropip(
     selenium_coverage: typing.Any,
     testserver_http: PyodideServerInfo,
-    run_from_server: ServerRunnerInfo,
 ) -> None:
     @run_in_pyodide(packages=["micropip"])  # type: ignore[misc]
     async def test_fn(
@@ -751,7 +743,6 @@ def test_open_close(
 @pytest.mark.webworkers
 @pytest.mark.without_jspi
 def test_break_worker_streaming(
-    selenium_coverage: typing.Any,
     testserver_http: PyodideServerInfo,
     run_from_server: ServerRunnerInfo,
 ) -> None:
@@ -944,6 +935,68 @@ def test_retries(
     pyodide_test(selenium_coverage, testserver_http.http_host, find_unused_port())
 
 
+def test_redirects(
+    selenium_coverage: typing.Any, testserver_http: PyodideServerInfo
+) -> None:
+    @run_in_pyodide  # type: ignore[misc]
+    def pyodide_test(selenium_coverage: typing.Any, host: str, port: int) -> None:
+        from urllib3 import request
+
+        redirect_url = f"http://{host}:{port}/redirect"
+        response = request("GET", redirect_url)
+        assert response.status == 200
+
+    pyodide_test(
+        selenium_coverage, testserver_http.http_host, testserver_http.http_port
+    )
+
+
+@pytest.mark.with_jspi
+def test_disabled_redirects(
+    selenium_coverage: typing.Any, testserver_http: PyodideServerInfo
+) -> None:
+    """
+    Test that urllib3 can control redirects in Node.js.
+    """
+
+    @run_in_pyodide  # type: ignore[misc]
+    def pyodide_test(selenium_coverage: typing.Any, host: str, port: int) -> None:
+        import pytest
+
+        from urllib3 import PoolManager, request
+        from urllib3.contrib.emscripten.fetch import _is_node_js
+        from urllib3.exceptions import MaxRetryError
+
+        if not _is_node_js():
+            pytest.skip("urllib3 does not control redirects in browsers.")
+
+        redirect_url = f"http://{host}:{port}/redirect"
+
+        with PoolManager(retries=0) as http:
+            with pytest.raises(MaxRetryError):
+                http.request("GET", redirect_url)
+
+            response = http.request("GET", redirect_url, redirect=False)
+            assert response.status == 303
+
+        with PoolManager(retries=False) as http:
+            response = http.request("GET", redirect_url)
+            assert response.status == 303
+
+        with pytest.raises(MaxRetryError):
+            request("GET", redirect_url, retries=0)
+
+        response = request("GET", redirect_url, redirect=False)
+        assert response.status == 303
+
+        response = request("GET", redirect_url, retries=0, redirect=False)
+        assert response.status == 303
+
+    pyodide_test(
+        selenium_coverage, testserver_http.http_host, testserver_http.http_port
+    )
+
+
 def test_insecure_requests_warning(
     selenium_coverage: typing.Any, testserver_http: PyodideServerInfo
 ) -> None:
@@ -970,36 +1023,34 @@ def test_insecure_requests_warning(
 
 @pytest.mark.webworkers
 def test_has_jspi_worker(
-    selenium_coverage: typing.Any,
     testserver_http: PyodideServerInfo,
     run_from_server: ServerRunnerInfo,
-    has_jspi: bool,
+    prefer_jspi: bool,
 ) -> None:
     worker_code = f"""
     import urllib3.contrib.emscripten.fetch
-    assert(urllib3.contrib.emscripten.fetch.has_jspi() == {has_jspi})
+    assert(urllib3.contrib.emscripten.fetch.has_jspi() == {prefer_jspi})
     """
 
     run_from_server.run_webworker(worker_code)
 
 
 def test_has_jspi(
-    selenium_coverage: typing.Any, testserver_http: PyodideServerInfo, has_jspi: bool
+    selenium_coverage: typing.Any, testserver_http: PyodideServerInfo, prefer_jspi: bool
 ) -> None:
     @run_in_pyodide
-    def pyodide_test(selenium, has_jspi):  # type: ignore[no-untyped-def]
+    def pyodide_test(selenium, prefer_jspi):  # type: ignore[no-untyped-def]
         import urllib3.contrib.emscripten.fetch
 
-        assert urllib3.contrib.emscripten.fetch.has_jspi() == has_jspi
+        assert urllib3.contrib.emscripten.fetch.has_jspi() == prefer_jspi
 
-    pyodide_test(selenium_coverage, has_jspi)
+    pyodide_test(selenium_coverage, prefer_jspi)
 
 
 @pytest.mark.with_jspi
 def test_timeout_jspi(
     selenium_coverage: typing.Any,
     testserver_http: PyodideServerInfo,
-    run_from_server: ServerRunnerInfo,
 ) -> None:
     @run_in_pyodide
     def pyodide_test(selenium, host, port):  # type: ignore[no-untyped-def]
@@ -1058,6 +1109,44 @@ def test_streaming_jspi(
     )
 
 
+# another streaming test - uses chunked read
+# and streaming to check that it works okay
+# (see https://github.com/urllib3/urllib3/issues/3555 )
+@pytest.mark.with_jspi
+def test_streaming2_jspi(
+    selenium_coverage: typing.Any, testserver_http: PyodideServerInfo
+) -> None:
+    bigfile_url = (
+        f"http://{testserver_http.http_host}:{testserver_http.http_port}/dripfeed"
+    )
+
+    @run_in_pyodide
+    def pyodide_test(selenium, host, port, bigfile_url):  # type: ignore[no-untyped-def]
+        from urllib3.connection import HTTPConnection
+        from urllib3.response import BaseHTTPResponse
+
+        conn = HTTPConnection(host, port)
+        conn.request("GET", bigfile_url, preload_content=False)
+        response = conn.getresponse()
+        assert isinstance(response, BaseHTTPResponse)
+        # get first data
+        all_data = response.read(32768)
+        # now get the rest in chunks
+        # to make sure that streaming works
+        # correctly even if the low level read doesn't
+        # always return a full buffer (which it doesn't)
+        while not response._response.body.closed:  # type: ignore[attr-defined]
+            all_data += response.read(32768)
+        assert len(all_data.decode("utf-8")) == 17825792
+
+    pyodide_test(
+        selenium_coverage,
+        testserver_http.http_host,
+        testserver_http.http_port,
+        bigfile_url,
+    )
+
+
 @pytest.mark.node_without_jspi
 def test_non_jspi_fail_in_node(
     selenium_coverage: typing.Any, testserver_http: PyodideServerInfo
@@ -1189,3 +1278,23 @@ def test_has_jspi_exception(
     pyodide_test(
         selenium_coverage, testserver_http.http_host, testserver_http.http_port
     )
+
+
+@run_in_pyodide  # type: ignore[misc]
+def test_pool_no_port(selenium_coverage: typing.Any) -> None:
+    from unittest.mock import patch
+
+    from urllib3 import HTTPConnectionPool
+    from urllib3.contrib.emscripten.request import EmscriptenRequest
+    from urllib3.contrib.emscripten.response import EmscriptenResponse
+
+    def send_request(request: EmscriptenRequest) -> EmscriptenResponse:
+        assert request.url == "http://example.com/"
+        return EmscriptenResponse(
+            status_code=200, headers={}, body=b"", request=request
+        )
+
+    with patch("urllib3.contrib.emscripten.connection.send_request", new=send_request):
+        pool = HTTPConnectionPool("example.com", maxsize=10, block=True)
+
+        pool.request("GET", "/")
