@@ -230,6 +230,10 @@ class TestHTTPHeaderDict:
         assert "cookie" not in d
         assert "COOKIE" not in d
 
+    def test_delitem_with_bytes_key(self, d: HTTPHeaderDict) -> None:
+        del d[b"cookie"]  # type: ignore[arg-type]
+        assert "cookie" not in d
+
     def test_add_well_known_multiheader(self, d: HTTPHeaderDict) -> None:
         d.add("COOKIE", "asdf")
         assert d.getlist("cookie") == ["foo", "bar", "asdf"]
@@ -318,6 +322,20 @@ class TestHTTPHeaderDict:
         d.add("b", "asdf")
         assert d.getlist("b") == ["asdf"]
 
+    def test_getlist_with_bytes_key(self, d: HTTPHeaderDict) -> None:
+        assert d.getlist(b"cookie") == ["foo", "bar"]  # type: ignore[call-overload]
+
+    def test_getitem_with_bytes(self, d: HTTPHeaderDict) -> None:
+        d["Content-Type"] = "application/json"
+        d.add("Content-Type", "charset=utf-8")
+        result = d[b"Content-Type"]  # type: ignore[index]
+        assert result == "application/json, charset=utf-8"
+
+    def test_contains_with_bytes(self, d: HTTPHeaderDict) -> None:
+        d["Content-Type"] = "application/json"
+        assert b"Content-Type" in d  # type: ignore[comparison-overlap]
+        assert b"X-Not-There" not in d  # type: ignore[comparison-overlap]
+
     def test_getlist_after_copy(self, d: HTTPHeaderDict) -> None:
         assert d.getlist("cookie") == HTTPHeaderDict(d).getlist("cookie")
 
